@@ -4,7 +4,7 @@ import threading
 from collections import Counter, namedtuple
 from dataclasses import dataclass, field, fields
 from time import monotonic
-from typing import Iterable, Iterator, NamedTuple, Optional, Type, Union
+from typing import Iterable, Iterator, NamedTuple, Optional, Set, Type, Union
 
 from clickhouse_pool import ChPool
 
@@ -130,10 +130,16 @@ class ClickhouseCubeBackend(CubeBackend):
         self._drop_table(cube)
         self._table_exists.pop(self.cube_to_table_name(cube), None)
 
-    def sync_storage(self, cube: Type[Cube], drop=False) -> None:
+    def sync_storage(self, cube: Type[Cube], drop=False) -> (bool, Set[str], Set[str]):
         """
         Assumes that the table already exists and makes sure columns are added or removed
         to match the cube definition.
+
+        :param cube: the cube class
+        :param drop: if True, drop columns that are not in the cube definition
+        :returns: (changed, added, removed) where changed is True if the table was changed;
+        added is a set of columns added; removed is a set of columns to be removed - regardless
+        of whether they were actually removed or not (depends on `drop`).
         """
         table_name = self.cube_to_table_name(cube)
         with self.pool.get_client() as client:
@@ -145,10 +151,12 @@ class ClickhouseCubeBackend(CubeBackend):
             cols_to_add = cube_col_names - db_col_names
             cols_to_remove = db_col_names - cube_col_names
 
+            changed = False
             if drop and cols_to_remove:
                 logger.debug(f"Dropping columns {cols_to_add} from table '{table_name}'")
                 for col in cols_to_remove:
                     client.execute(f"ALTER TABLE {self.database}.{table_name} DROP COLUMN {col}")
+                changed = True
 
             if cols_to_add:
                 logger.debug(f"Adding columns {cols_to_add} to table '{table_name}'")
@@ -156,6 +164,8 @@ class ClickhouseCubeBackend(CubeBackend):
                     dim = cube._dimensions.get(col) or cube._metrics.get(col)
                     spec = self._dim_spec(dim)
                     client.execute(f"ALTER TABLE {self.database}.{table_name} ADD COLUMN {spec}")
+                changed = True
+            return changed, cols_to_add, cols_to_remove
 
     def store_records(self, cube: Type[Cube], records: Iterable[NamedTuple]):
         with self.pool.get_client() as client:
